// This program reads the database for the adventure game and builds a .h
// file which can be included in the adventure code, such that the database
// is already loaded into the arrays, without it having to be read in.

#include "cave.h"


///////////////////////////////////////////////////////////////////////////////
// Returns the number of digits in an integer (+1 if the integer is negative).
///////////////////////////////////////////////////////////////////////////////

int numdigits(int n)
{
  int dec=10;
  int count=1;

  if (n < 0) {
    count++;
    n = -n;
  }

  while (count < 10 && dec <= n) {
    dec *= 10;
    count++;
  }
  return count;
}


///////////////////////////////////////////////////////////////////////////////
// Dumps an array into file pointed to by fp
///////////////////////////////////////////////////////////////////////////////

void dump_array(FILE* fp, int size, int array[])
{
  int charsonline=0;

  for (int i=0; i<=size; i++) {
    int n = array[i];
    if (charsonline+numdigits(n)+1 > 72) {
      fputs("\n  ",fp);
      charsonline=0;
    }
    fprintf(fp,"%d%c", n, i==size?'\n':',');
    charsonline += numdigits(n) + 1;
  }
  fputs("};\n\n", fp);
}


///////////////////////////////////////////////////////////////////////////////
// Reads the adventure database in and then builds a .h file with the data
///////////////////////////////////////////////////////////////////////////////

int main()
{
  FILE* fp;
  int I;
  int K;
  int VERB;

  read_db();

  fp = fopen("db.h", "wa");
  if (fp == NULL) {
    perror("Couldn't open db.h");
    exit(1);
  }

  fputs("char LINES[] = {\n", fp);
  K=0;
  while (K<LINUSE) {
    fputs("  \"", fp);
    for (I=0; I<72; I++) {
      if (K<LINUSE) {
	char c = LINES[K++];
	if (c == '"') {
	  fputs("\\\"", fp);
	  I++;
	}
	else if (c == '\n') {
	  fputs("\\n", fp);
	  I++;
	}
	else if (c == '\0') {
	  fputs("\\0", fp);
	  I++;
	}
	else if (isprint(c)) {
	  fputc(c, fp);
	}
	else {   // it's not good
	  printf("Unknown character in LINUSE: %02x\n", c);
	  exit(1);
	}
      }
    }
    fputs("\"\n", fp);
  }
  fputs("};\n\n", fp);

  // Build arrays related to locations

  for (K=LOCSIZ-1; K>0; K--) {
    if (LTEXT[K]!=0) break;
  }

  // long descriptions

  fprintf(fp, "const int LOCSIZ=%d;\n\n", K+1);

  fprintf(fp, "int LTEXT[LOCSIZ] = {\n  ");
  dump_array(fp, K, LTEXT);

  // short descriptions

  fprintf(fp, "int STEXT[LOCSIZ] = {\n  ");
  dump_array(fp, K, STEXT);

  fprintf(fp, "int KEY[LOCSIZ] = {\n  ");
  dump_array(fp, K, KEY);

  fprintf(fp, "int COND[LOCSIZ] = {\n  ");
  dump_array(fp, K, COND);


  // Travel

  fprintf(fp, "int TRAVEL[%d] = {\n  ", TRVS+1);
  dump_array(fp, TRVS, TRAVEL);

  // Vocabulary

  fprintf(fp, "const int TABSIZ=%d;\n\n", TABNDX);

  fputs("int KTAB[TABSIZ+1] = {\n  ", fp);
  dump_array(fp, TABNDX, KTAB);

  fputs("char ATAB[TABSIZ*5+1] = {\n", fp); // +1 allows room for null at the
	                                    // end of the initializer string.
  K=0;
  while (K<TABNDX*5) {
    fputs("  \"", fp);
    for (I=0; I<72; I++) {
      if (K<TABNDX*5) {
	char c = ATAB[K++];
	if (c == '\0') {
	  fputs("\\0", fp);
	  I++;
	}
	else if (c == '"') {
	  fputs("\\\"", fp);
	  I++;
	}
	else {
	  fputc(c, fp);
	}
      }
    }
    fputs("\"\n", fp);
  }
  fputs("};\n\n", fp);


  for (I=1; I<TABNDX; I++) if (KTAB[I]/1000==2) VERB=KTAB[I]-2000;
  printf("   %d OF %d action verbs\n", VERB, VRBSIZ);

  fprintf(fp, "int ACTSPK[35] = {\n  ");
  dump_array(fp, 34, ACTSPK);

  // Objects

  for (K=OBJSIZ-1; K>0; K--) {
    if (PTEXT[K]!=0) break;
  }

  fprintf(fp, "const int OBJSIZ=%d;\n\n", K+1);

  fprintf(fp, "int PTEXT[OBJSIZ] = {\n  ");
  dump_array(fp, K, PTEXT);

  fprintf(fp, "int PINDX[%d] = {\n  ", PRPUSE+1);
  dump_array(fp, PRPUSE, PINDX);

  fprintf(fp, "int PLAC[OBJSIZ] = {\n  ");
  dump_array(fp, K, PLAC);

  fprintf(fp, "int FIXD[OBJSIZ] = {\n  ");
  dump_array(fp, K, FIXD);


  // Random Messages

  for (K=RTXSIZ-1; K>0; K--) {
    if (RTEXT[K]!=0) break;
  }

  fprintf(fp, "int RTEXT[%d] = {\n  ", K+1);
  dump_array(fp, K, RTEXT);

  // Class Messages

  fprintf(fp, "const int CLSSES=%d;\n", CLSSES);

  fprintf(fp, "int CTEXT[CLSSES] = {\n  ");
  dump_array(fp, CLSSES-1, CTEXT);

  fprintf(fp, "int CVAL[CLSSES] = {\n  ");
  dump_array(fp, CLSSES-1, CVAL);


  // Hints

  fprintf(fp, "const int HNTMAX=%d;\n", HNTMAX);
  fprintf(fp, "const int HNTSIZ=%d;\n", HNTSIZ);
  fputs("struct {\n"
	"  int TRNS;\n"
	"  int PNTS;\n"
	"  int QSTN;\n"
	"  int HINT;\n"
	"} HINTS[HNTSIZ] = {\n", fp);

  for (I=0; I<=HNTMAX; I++) {
    fprintf(fp, "  { %d, %d, %d, %d }%s\n",
	    HINTS[I].TRNS, HINTS[I].PNTS, HINTS[I].QSTN, HINTS[I].HINT,
	    I==HNTMAX ? "":",");
  }
  fputs("};\n\n", fp);

  // Magic messages

  for (K=MAGSIZ-1; K>0; K--) {
    if (MTEXT[K]!=0) break;
  }

  fprintf(fp, "int MTEXT[%d] = {\n  ", K+1);
  dump_array(fp, K, MTEXT);

  if (ferror(fp)) {
    perror("Error occurred creating file");
  }

  fclose(fp);
}


////////////////////////////////////////////////////////////////////////////
// BUG
//
// THE FOLLOWING CONDITIONS ARE CURRENTLY CONSIDERED FATAL BUGS.  NUMBERS < 20
// ARE DETECTED WHILE READING THE DATABASE; THE OTHERS OCCUR AT "RUN TIME".
//	0	MESSAGE LINE > 70 CHARACTERS
//	1	NULL LINE IN MESSAGE
//	2	TOO MANY WORDS OF MESSAGES
//	3	TOO MANY TRAVEL OPTIONS
//	4	TOO MANY VOCABULARY WORDS
//	5	REQUIRED VOCABULARY WORD NOT FOUND
//	6	TOO MANY RTEXT OR MTEXT MESSAGES
//	7	TOO MANY HINTS
//	8	LOCATION HAS COND BIT BEING SET TWICE
//	9	INVALID SECTION NUMBER IN DATABASE
//	20	SPECIAL TRAVEL (500>L>300) EXCEEDS GOTO LIST
//	21	RAN OFF END OF VOCABULARY TABLE
//	22	VOCABULARY TYPE (N/1000) NOT BETWEEN 0 AND 3
//	23	INTRANSITIVE ACTION VERB EXCEEDS GOTO LIST
//	24	TRANSITIVE ACTION VERB EXCEEDS GOTO LIST
//	25	CONDITIONAL TRAVEL ENTRY WITH NO ALTERNATIVE
//	26	LOCATION HAS NO TRAVEL ENTRIES
//	27	HINT NUMBER EXCEEDS GOTO LIST
//	28	INVALID MONTH RETURNED BY DATE FUNCTION
////////////////////////////////////////////////////////////////////////////

void BUG(int NUM)
{
    printf(" Fatal error, see source code for interpretation.\n"
	   " Probable cause: erroneous info in database.\n"
	   " Error code = %d\n", NUM);
    exit(1);
}

